import{i as e,n as t,o as n,r}from"./use-api-BAbm8GVs.js";const i=r({endpointKey:`GET_CURRENT_USER`,responseType:{},headers:{"Cache-Control":`no-store`}}),a=r({endpointKey:`USERS_RESOURCE`,responseType:[]}),o=r({endpointKey:`USERS_RESOURCE`,responseType:[],queryParams:{service:!0}}),s=t({endpointKey:`GET_USER_DETAILS`,responseType:{}}),c=async t=>e(n.USERS_RESOURCE,{method:`POST`,body:JSON.stringify(t)}),l=async t=>e(n.USERS_RESOURCE,{method:`DELETE`,body:JSON.stringify({username:t})});export{i as a,a as i,l as n,s as o,o as r,c as t};