import{r as e,t}from"./jsx-runtime-DAs1UGHr.js";import"./react-DxF41JXl.js";import"./use-runtime-config-D_-Z8FUB.js";import"./use-api-BAbm8GVs.js";import"./use-user-BydImGZB.js";import"./user-service-Cq_o-JT5.js";import"./toast-context-val-CcPDsXTx.js";import"./page-container-DV9_twon.js";import"./entity-service-D6-I4F0h.js";import"./use-all-experiments-2M5Vv300.js";import"./search-input-DVDKmJAY.js";import"./icon-button-8v1zbWfj.js";import"./use-toast-CpnYXdO8.js";import"./grant-permission-modal-Du7KzdTv.js";import{t as n}from"./shared-permissions-page-B1KPKpf2.js";import"./use-user-prompt-permissions-XTSblT8y.js";import"./use-all-models-ByKExnMB.js";import"./use-all-prompts-CDSoqi-1.js";var r=e(t(),1);function i({type:e}){return(0,r.jsx)(n,{type:e,baseRoute:`/service-accounts`,entityKind:`user`})}export{i as default};