import{n as e,r as t}from"./use-api-BAbm8GVs.js";const n=t({endpointKey:`ALL_GROUPS`,responseType:[]}),r=t({endpointKey:`ALL_EXPERIMENTS`,responseType:[]}),i=t({endpointKey:`ALL_MODELS`,responseType:[]}),a=t({endpointKey:`ALL_PROMPTS`,responseType:[]}),o=e({endpointKey:`EXPERIMENT_USER_PERMISSIONS`,responseType:[]}),s=e({endpointKey:`USER_EXPERIMENT_PERMISSIONS`,responseType:[]}),c=e({endpointKey:`USER_MODEL_PERMISSIONS`,responseType:[]}),l=e({endpointKey:`USER_PROMPT_PERMISSIONS`,responseType:[]}),u=e({endpointKey:`USER_EXPERIMENT_PATTERN_PERMISSIONS`,responseType:[]}),d=e({endpointKey:`USER_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),f=e({endpointKey:`USER_PROMPT_PATTERN_PERMISSIONS`,responseType:[]}),p=e({endpointKey:`MODEL_USER_PERMISSIONS`,responseType:[]}),m=e({endpointKey:`PROMPT_USER_PERMISSIONS`,responseType:[]}),h=e({endpointKey:`GROUP_EXPERIMENT_PERMISSIONS`,responseType:[]}),g=e({endpointKey:`GROUP_MODEL_PERMISSIONS`,responseType:[]}),_=e({endpointKey:`GROUP_PROMPT_PERMISSIONS`,responseType:[]}),v=e({endpointKey:`EXPERIMENT_GROUP_PERMISSIONS`,responseType:[]}),y=e({endpointKey:`MODEL_GROUP_PERMISSIONS`,responseType:[]}),b=e({endpointKey:`PROMPT_GROUP_PERMISSIONS`,responseType:[]}),x=e({endpointKey:`GROUP_EXPERIMENT_PATTERN_PERMISSIONS`,responseType:[]}),S=e({endpointKey:`GROUP_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),C=e({endpointKey:`GROUP_PROMPT_PATTERN_PERMISSIONS`,responseType:[]});export{c as S,u as _,v as a,f as b,h as c,_ as d,g as f,m as g,b as h,a as i,S as l,p as m,n,o,y as p,i as r,x as s,r as t,C as u,s as v,l as x,d as y};