import{t as e}from"./use-api-BAbm8GVs.js";import{a as t}from"./user-service-Cq_o-JT5.js";function n(){let{data:n,isLoading:r,error:i,refetch:a}=e(t);return{currentUser:n,isLoading:r,error:i,refresh:a}}export{n as t};