import{r as e,t}from"./jsx-runtime-DAs1UGHr.js";import{t as n}from"./react-DxF41JXl.js";import{o as r}from"./chunk-EPOLDU6W-CNaCjKeI.js";import{n as i}from"./use-runtime-config-D_-Z8FUB.js";import{_ as a,j as o}from"./use-api-BAbm8GVs.js";n();var s=e(t(),1);function c({entityId:e,route:t,buttonText:n}){let c=r(),l=`${i().basePath}/${t}/${e}`;return(0,s.jsx)(o,{onClick:e=>{e.stopPropagation(),c(l)},icon:a,className:`gap-1`,children:n})}export{c as t};